#include "utils/directories.hpp"
#include <iostream>
#include <fstream>
#include <boost/filesystem.hpp>
#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/ini_parser.hpp>
#include "config.hpp"
#include "utils/envmanager.hpp"

namespace fs = boost::filesystem;
namespace pt = boost::property_tree;

namespace dropshell {


config *get_global_config() {
    static config *gConfig = new config();
    return gConfig;
}

config::config() {
}
config::~config() {
}

bool config::load_config() {
    std::string config_path = get_local_dropshell_config_path();
    if (config_path.empty() || !fs::exists(config_path))
        return false;

    envmanager config_env(config_path);
    if (!config_env.load())
        return false;

    std::string mDirectories = config_env.get_variable_substituted("local.config.directories");
    if (mDirectories.empty())
            return false;   

    // Split the directories string into a vector of strings
    mLocalConfigPaths = string2multi(mDirectories);

    //std::cout << "Local config path: " << mLocalConfigPath << std::endl;
    return true;
}

void config::save_config()
{
    if (mLocalConfigPaths.empty())
    {
        std::cerr << "Warning: Unable to save configuration file, as DropShell has not been initialised."<< std::endl;
        std::cerr << "Please run 'dropshell init <path>' to initialise DropShell." << std::endl;
        return;
    }

    std::string config_path = get_local_dropshell_config_path();
    envmanager config_env(config_path);

    config_env.set_variable("local.config.directories", multi2string(mLocalConfigPaths));
    config_env.save();
}

bool config::is_config_set() const
{   
    return !mLocalConfigPaths.empty();
}


const std::vector<std::string> & config::get_local_config_directories() const
{
    return mLocalConfigPaths;
}

bool config::add_local_config_directory(const std::string &path)
{
    if (path.empty())
        return false;

    // Convert to canonical path
    fs::path abs_path = fs::canonical(path);
    
    // The directory must exist
    if (!fs::exists(abs_path)) {
        std::cerr << "Error: The local config directory does not exist: " << abs_path.string() << std::endl;
        return false;
    }

    // Add to config paths if not already there
    std::string path_str = abs_path.string();
    if (std::find(mLocalConfigPaths.begin(), mLocalConfigPaths.end(), path_str) == mLocalConfigPaths.end()) {
        mLocalConfigPaths.push_back(path_str);
        return true;
    }

    std::cerr << "Warning: The local config directory is already registered: " << path_str << std::endl;
    std::cerr << "No changes made to the DropShell configuration." << std::endl;
    return false;
}

} // namespace dropshell 