#include "server_env.hpp"
#include "utils/envmanager.hpp"
#include "utils/directories.hpp"
#include <boost/filesystem.hpp>
#include <iostream>
#include <memory>

namespace dropshell {

bool server_env::is_valid() const {
    return mValid;
}

server_env::server_env(const std::string& server_name) : mValid(false) {
    if (server_name.empty())
    {
        std::cerr << "Warning: Server name is empty, passed to server_env constructor." << std::endl;
        return;
    }
        
    // Construct the full path to server.env
    std::string env_path = get_local_server_env_path(server_name);
    
    // Check if file exists
    if (!boost::filesystem::exists(env_path)) {
        throw std::runtime_error("Server environment file not found: " + env_path);
    }

    try {
        // Use envmanager to handle the environment file
        m_env_manager = std::unique_ptr<envmanager>(new envmanager(env_path));
        m_env_manager->load();
        
        // Get all variables
        m_env_manager->get_all_variables_substituted(variables);

        // Verify required variables exist
        for (const auto& var : {"SSH_HOST", "SSH_USER", "SSH_PORT", "DROPSHELL_DIR"}) {
            if (variables.find(var) == variables.end()) {
                // Print the variables identified in the file
                std::cout << "Variables identified in the file:" << std::endl;
                for (const auto& v : variables) {
                    std::cout << "    " << v.first << std::endl;
                }
                throw std::runtime_error("Missing required variable: " + std::string(var));
            }
        }
        mValid = true;

    } catch (const std::exception& e) {
        throw std::runtime_error("Failed to parse server environment file: " + std::string(e.what()));
    }
}

std::string server_env::get_variable(const std::string& name) const {
    if (!m_env_manager) {
        return "";
    }
    return m_env_manager->get_variable_substituted(name);
}

const std::map<std::string, std::string> &server_env::get_variables() const
{
    return variables;
}

std::string server_env::get_SSH_HOST() const {
    return get_variable("SSH_HOST");
}

std::string server_env::get_SSH_USER() const {
    return get_variable("SSH_USER");
}

std::string server_env::get_SSH_PORT() const {
    return get_variable("SSH_PORT");
}

std::string server_env::get_DROPSHELL_DIR() const {
    return get_variable("DROPSHELL_DIR");
}

} // namespace dropshell 